<?php

namespace Drupal\session_inspector\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\session_inspector\Plugin\BrowserFormatManager;
use Drupal\session_inspector\Plugin\HostnameFormatManager;

/**
 * A form for configuring the session inspector module.
 *
 * @package Drupal\session_inspector\Form
 */
class SessionInspectorConfigForm extends ConfigFormBase {

  /**
   * The browser format plugin manager.
   *
   * @var \Drupal\session_inspector\Plugin\BrowserFormatManager
   */
  protected $browserFormatManager;

  /**
   * The hostname format plugin manager.
   *
   * @var \Drupal\session_inspector\Plugin\HostnameFormatManager
   */
  protected $hostnameFormatManager;

  /**
   * Constructs a SessionInspectorConfigForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\session_inspector\Plugin\BrowserFormatManager $browser_format_manager
   *   The browser format plugin manager.
   * @param \Drupal\session_inspector\Plugin\HostnameFormatManager $hostname_format_manager
   *   The hostname format plugin manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, BrowserFormatManager $browser_format_manager, HostnameFormatManager $hostname_format_manager) {
    parent::__construct($config_factory);

    $this->browserFormatManager = $browser_format_manager;
    $this->hostnameFormatManager = $hostname_format_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.session_inspector.browser_format'),
      $container->get('plugin.manager.session_inspector.hostname_format')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'session_inspector.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'session_inspector_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('session_inspector.settings');

    // Set up the browser format plugin options.
    $browserFormatPluginDefinitions = $this->browserFormatManager->getDefinitions();

    $browserFormatOptions = [];
    foreach ($browserFormatPluginDefinitions as $pluginName => $pluginDefinition) {
      /** @var \Drupal\Core\StringTranslation\TranslatableMarkup $pluginDefinition */
      $pluginDefinitionName = $pluginDefinition['name'];
      $browserFormatOptions[$pluginName] = $pluginDefinitionName->render();
    }

    $form['browser_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Browser format'),
      '#default_value' => $config->get('browser_format') ?? 'basic',
      '#options' => $browserFormatOptions,
    ];

    // Set up the hostname format plugin options.
    $hostnameFormatPluginDefinitions = $this->hostnameFormatManager->getDefinitions();

    $hostnameFormatOptions = [];
    foreach ($hostnameFormatPluginDefinitions as $pluginName => $pluginDefinition) {
      /** @var \Drupal\Core\StringTranslation\TranslatableMarkup $pluginDefinition */
      $pluginDefinitionName = $pluginDefinition['name'];
      $hostnameFormatOptions[$pluginName] = $pluginDefinitionName->render();
    }

    $form['hostname_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Hostname format'),
      '#default_value' => $config->get('hostname_format') ?? 'basic',
      '#options' => $hostnameFormatOptions,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('session_inspector.settings')
      ->set('hostname_format', $form_state->getValue('hostname_format'))
      ->set('browser_format', $form_state->getValue('browser_format'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
